/*---------------------------------------------------------------------

mynoise.h

This file is part of CytoPacq

Copyright (C) 2007,2008 -- Centre for Biomedical Image Analysis (CBIA)

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda

Description: Generation of various types of noise.

-------------------------------------------------------------------------*/

#ifndef _CAMERA_NOISE_
#define _CAMERA_NOISE_

#include <i3d/image3d.h>

//---------------------------------------------------------------------------
// photon noise
//---------------------------------------------------------------------------
template <class ELECTRONS> 
void ApplyPoissonNoise(i3d::Image3d<ELECTRONS> &img); 

//---------------------------------------------------------------------------
// read out
//---------------------------------------------------------------------------
template <class ELECTRONS> 
void ApplyReadoutNoise(i3d::Image3d<ELECTRONS> &img, 
							  float sigma); 

//---------------------------------------------------------------------------
// hot/cold pixels
//---------------------------------------------------------------------------
template <class ELECTRONS> 
void SetAmountOfElectrons(i3d::Image3d<ELECTRONS> &img, 
								  size_t x, 
								  size_t y,
								  ELECTRONS amount); 

//---------------------------------------------------------------------------
// dark current
//---------------------------------------------------------------------------
template <class ELECTRONS>
void AddDarkCurrent(i3d::Image3d<ELECTRONS> &image,  
						  float el_added, 
						  float ac_time); 

#endif
