/*---------------------------------------------------------------------

plug.h

This file is part of CytoPacq

Copyright (C) 2007,2008 -- Centre for Biomedical Image Analysis (CBIA)

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda

Description: A module responsible for plugins loading.

-------------------------------------------------------------------------*/

#ifndef _MYPLUG_
#define _MYPLUG_

#include <vector>
#include <string>

#ifdef _WIN32
	#include <windows.h>
	#define T_HANDLE HMODULE
#else
	#include <dlfcn.h>
	#define T_HANDLE void*
#endif

/*
 * Search the given directory for plugin containing the identifier
 * specified in the argument + open it.
 */
T_HANDLE FindAndOpenPlugin(const char* identifier, 
									const std::vector<std::string> &);

/*
 * Open the plugin of the given filename.
 */
T_HANDLE OpenPlugin(const char* fn);

/*
 * Fetch the symbol from the openned plugin
 */
void *GetSymbolFromPlugin(T_HANDLE fhandle, const char *symbol_name);

/*
 * Close the plugin
 */
void ReleasePlugin(T_HANDLE fhandle);

/*
 * Find all the possible plugins in the give ndirectory
 */
int SearchForPlugins(std::vector<std::string> &plugin_list,
							const char *directory);

#endif
