/*---------------------------------------------------------------------

add_shape.h

This file is part of CytoPacq

Copyright (C) 2007-2013 -- Centre for Biomedical Image Analysis (CBIA)

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda

Description: The functions that can: merge binary images, add some primitives (ellipsoids, cylinders, snakes).

-------------------------------------------------------------------------*/

#ifndef _ADD_SHAPE_
#define _ADD_SHAPE_

#include <vector>
#include <i3d/image3d.h>

void MergeBinaryImages(i3d::Image3d<bool> &img_acc, 
							  const i3d::Image3d<bool> &img_added,
							  i3d::Vector3d<size_t> shift = 0);

void MergeBinaryImages(i3d::Image3d<bool> &img_acc, 
							  const i3d::Image3d<bool> &img_added,
							  i3d::Vector3d<float> shift = 0);

template <class T> 
void GetLabelMask(const i3d::Image3d<T> &img,
						T label,
						i3d::Image3d<bool> &mask);

template <class T> 
bool AddNewLabel(i3d::Image3d<T> &img,
					  const i3d::Image3d<bool>& mask,
					  bool ignore_overlaps = false);

template <class T> 
void StickToImage(i3d::Image3d<T> &,
						const i3d::Image3d<T> &, 
						const i3d::Image3d<bool> &);

template <class VOXEL> void AddEllipsoid(
		i3d::Image3d<VOXEL> &img,
		const float centerX,
		const float centerY,
		const float centerZ,
		const float radiusX,
		const float radiusY,
		const float radiusZ,
		const float angle_xy = 0,
		const float angle_xz = 0,
		const VOXEL foreground_intensity = std::numeric_limits<VOXEL>::max());


template <class VOXEL> void AddCylinder(
		i3d::Image3d<VOXEL> &img,
		const float bottomX,
		const float bottomY,
		const float bottomZ,
		const float topX,
		const float topY,
		const float topZ,
		const float diameter,
		const VOXEL foreground_intensity = std::numeric_limits<VOXEL>::max());


template <class VOXEL> void AddSnake(
		i3d::Image3d<VOXEL> &img,
		std::vector<i3d::Vector3d<float>/**/> &nodes,
		const float diameter,
		const VOXEL foreground_intensity = std::numeric_limits<VOXEL>::max());

#endif

