#!/bin/bash

# check the number of in-arguments
if [ $# != 12 ]; 
then
		  echo "Bad usage of this script.";
		  exit 1;
fi

# This line will be necessary until we update the system on Alfa server
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/usr/local/lib/

# prepare the required variables
DATA_DIR=$1
KEY=$2
WORKING_DIR=$DATA_DIR"/"$KEY
CFG_FILE=$WORKING_DIR"/"$3
PHANTOM_IMG=$WORKING_DIR"/"$4
PHANTOM_PREVIEW=$WORKING_DIR"/"$5
LABELED_IMG=$WORKING_DIR"/"$6
BLURRED_IMG=$WORKING_DIR"/"$7
BLURRED_PREVIEW=$WORKING_DIR"/"$8
FINAL_IMG=$WORKING_DIR"/"$9
FINAL_PREVIEW=$WORKING_DIR"/"${10}
ERROR_LOG=$WORKING_DIR"/"${11}
WARNING_LOG=$WORKING_DIR"/"${12}

BIN_PATH=`dirname "$0"`

# ----------------------- tidy up the database --------------------------------
sh $BIN_PATH/run-db-cleanup.sh $DATA_DIR
# ----------------------- the database is tidy now --------------------------

# store the new process IP into database table
mysql -e "insert into processes set pid=\"$KEY\", start=now()"

# This process was started by web-interface on another server. The
# web interface is waiting until the new row in created in the database.
# As soon as it is done, it adds email and ip address of the user.
# Here we wait for completion of this two tasks.

# synchronize with web-interface 
TO_ADDRESS="NULL"
while [ $TO_ADDRESS = "NULL" ]
do
	TO_ADDRESS=`mysql -B -N -e "select email from processes where pid=\"$KEY\""`
	sleep 1s
done

VERSION="NULL"
while [ $VERSION = "NULL" ]
do
	VERSION=`mysql -B -N -e "select version from processes where pid=\"$KEY\""`
	sleep 1s
done

# ----------------------- start cytogen ----------------------------------
# make a record in database
mysql -e "update processes set cytogen_status=\"running\" where pid=\"$KEY\""

# compute
$BIN_PATH/3d-cytogen -c $CFG_FILE -p $PHANTOM_IMG -l $LABELED_IMG -e $ERROR_LOG -w $WARNING_LOG -d $BIN_PATH/../plugins

# has the computation finished with success?
if [ $? -ne 0 ] 
then
	mysql -e "update processes set cytogen_status=\"failed\" where pid=\"$KEY\""
	$BIN_PATH/notify_failed.sh $TO_ADDRESS $KEY $DATA_DIR $VERSION
	exit 1
fi

# ############################
# In case of success, proceed
# ############################

# find out which z-slice is the brightest one and store the value
# in the database
MAX_SLICE=`$BIN_PATH/get_max_slice_order $PHANTOM_IMG`
mysql -e "update processes set max_slice=$MAX_SLICE where pid=\"$KEY\""

# make preview image
$BIN_PATH/make_preview $PHANTOM_IMG $MAX_SLICE $PHANTOM_PREVIEW 
if [ -e $WARNING_LOG ] 
then
	mysql -e "update processes set cytogen_status=\"waiting\" where pid=\"$KEY\""
	TIME_SHIFT=`mysql -B -N -e "select unix_timestamp()-unix_timestamp(start) from processes where pid=\"$KEY\""`
	$BIN_PATH/notify_waiting.sh $TO_ADDRESS $KEY $DATA_DIR $VERSION $TIME_SHIFT
else
	mysql -e "update processes set cytogen_status=\"passed\" where pid=\"$KEY\""
	$BIN_PATH/proceed_with_simulation.sh $1 $2 $3 $4 $7 $8 $9 ${10} ${11} ${12}
fi

# ------------------------ stop cytogen ------------------------------------

