/*---------------------------------------------------------------------

entropy.cc

This file is part of CytoPacq

Copyright (C) 2007-2013 -- Centre for Biomedical Image Analysis (CBIA)

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda

Description: The evaluation of central moments over the list of images. The input is the list of images and the highest order of the central moment that the user wants to evaluate. The output is mean and variance for each central moment (up to the selected highest order).

-------------------------------------------------------------------------*/

#include "i3d/image3d.h"
#include "i3d/histogram.h"

using namespace std;
using namespace i3d;

typedef GRAY16 TYPE;

const size_t gmax = std::numeric_limits<TYPE>::max();



int main(int argc, char* argv[]) 
{
	if (argc<2)
	{
		cerr << "Usage: " << argv[0] << " <image>" << endl;
		cerr << " image - image file" << endl;
		exit(1);
	}

	Image3d<TYPE> img(argv[1]);


	Histogram hist(gmax);
	IntensityHist(img, hist);

	size_t count = img.GetImageSize();

	double sum = 0.0;
	double coef = 1.0 / log(2.0);

	for (size_t i=0; i<gmax; i++)
	{
		 double value = static_cast<double>(hist[i]);

		 if (hist[i] > 0)
		 {
			  value /= count;

			  sum += value * coef * log(value);
		 }
	}

	cout << (-1.0)*sum << endl;

	return 0;
}
