/*---------------------------------------------------------------------

moments.cc

This file is part of CytoPacq

Copyright (C) 2007-2013 -- Centre for Biomedical Image Analysis (CBIA)

CytoPacq is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

CytoPacq is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with CytoPacq. If not, see <http://www.gnu.org/licenses/>.

Author: David Svoboda

Description: The evaluation of central moments over the list of images. The input is the list of images and the highest order of the central moment that the user wants to evaluate. The output is mean and variance for each central moment (up to the selected highest order).

-------------------------------------------------------------------------*/

#include "i3d/image3d.h"

#define sqr(x) ((x)*(x))

using namespace std;
using namespace i3d;

double up(double base, size_t ex)
{
	if (ex == 0)
		return 1.0;

	return base * (up(base,ex-1));
}

void TypeMoments(size_t level, double moment)
{
	// cout << level << ". moment: " << moment1 << endl;
	cout << moment << "   ";
}

template <class VOXEL> void ComputeMoments(Image3d<VOXEL> & inputImg, size_t levels)
{
	double sum = 0.0, mean;


	// the first moment (mean)
	for (size_t i=0; i<inputImg.GetImageSize(); i++)
		sum += inputImg.GetVoxel(i);

	mean = sum / inputImg.GetImageSize();

	for (size_t l=2; l<levels+1; l++)
	{
		sum = 0.0;

		// the l-th moment
		for (size_t i=0; i<inputImg.GetImageSize(); i++)
			sum += up((inputImg.GetVoxel(i) - mean),l);

		double moment1 = pow(sum/inputImg.GetImageSize(), 1.0/l);
		
		TypeMoments(l, moment1);
	}
	cout << endl;
	

}

int main(int argc, char* argv[]) 
{
	if (argc<2)
	{
		cerr << "Usage: "<< argv[0] << " <image> [<level>=3]" << endl;
		cerr << " image - image file" << endl;
		cerr << " level - how many moments to compute (optional)" << endl;
		exit(1);
	}

	size_t levels = (argc==3) ? atoi(argv[2]) : 3 ;
	
	i3d::ImgVoxelType vt = i3d::ReadImageType(argv[1]);

    switch (vt)
    {
    case i3d::Gray8Voxel:
        {
	    Image3d<GRAY8> inputImg(argv[1]);
	    ComputeMoments(inputImg,levels);
            break;
        }
    case i3d::Gray16Voxel:
        {
	    Image3d<GRAY16> inputImg(argv[1]);
	    ComputeMoments(inputImg,levels);
            break;
        }

    default:
        std::cerr << "Other voxel types not supported" << std::endl;
    }
    return 0;

	return 0;
}
